#include "dumpacct.h"

/*
 * dumpacct.c
 *
 * program that dumps the contents of an account
 */

void main(int argc, char *argv[])
{
  int amount, md_counter, d_counter, start_date, end_date, fiscstart, fiscend;
  char *acctnum, *analnum, *acctnam;
  char minibuff[513];
  char *pgBeginDatum;
  char *pgEndDatum;
  char *desigBuff;

  pgBeginDatum = malloc(11);
  pgEndDatum = malloc(11);
  desigBuff = malloc(46);

  if ((argc < 3) || (argc == 4)) /* ambiguous */ {
    fprintf(stderr, "Usage: %s acct anal [start end]\n", argv[0]);
    fprintf(stderr, "where acct is three-digit account number\n");
    fprintf(stderr, "      anal is two-digit analytical extension (** means cumulative)\n");
    fprintf(stderr, "      start is start date in format YYYYMMDD\n");
    fprintf(stderr, "      end is end date in format YYYYMMDD\n");
    exit(0);
  } else { /* valid command-line parameters */
    initBackend();                /* initialize the backend connection */
    /* get the fiscal year range */
    queryFiscalYear(&fiscstart, &fiscend);
    if (argc > 4) { /* the user has provided both dates */
      start_date = atoi(argv[3]);
      end_date = atoi(argv[4]);
      /*
       * check validity of dates
       */
      if (!valiDate(start_date)) {
	fprintf(stderr, "Invalid starting date.\n");
	exit(0);
      }
      if (!valiDate(end_date)) {
	fprintf(stderr, "Invalid ending date.\n");
	exit(0);
      }
      if (start_date > end_date) {
	fprintf(stderr, "Start date must be less than or equal to end date!\n");
	exit(0);
      }
      if ((start_date < fiscstart) ||
	  (end_date > fiscend)) {
	fprintf(stderr, "Date range must fall within fiscal year.\n");
	exit(0);
      }
      /* OK -- the dates are valid */
    } else { /* the user didn't provide any dates */
      /* use the fiscal year as default
       */
      start_date = fiscstart;
      end_date = fiscend;
    }
  }

  /* check validity of supplied account number and then copy it to 
   * the local buffers acctnum and analnum so that if no analytical
   * number is supplied, analnum carries the default value of "--"
   */
  if ((strlen(argv[1]) != 3) ||
      (strlen(argv[2]) != 2)) {
    fprintf(stderr, "Invalid account number.\n");
    exit(0);
  }
  acctnum = malloc(4);
  analnum = malloc(3);
  acctnum = strcpy(acctnum, argv[1]);
  analnum = strcpy(analnum, argv[2]);

  acctnam = malloc(61);
  if ((acctnam = acctQuery(acctnam, acctnum, analnum)) == NULL) {
    fprintf(stderr, "The account you specified is not in the chart of accounts.\n");
    exit_nicely();
  } else {
    desigBuff=strncpy(desigBuff, acctnam, 41);
    printf("%s %s: %-41s: Od %d do %d\n", acctnum, analnum, desigBuff, start_date, end_date);
  }

  md_counter = d_counter = 0;
  if (!strcmp(analnum, "--")) {
    sprintf(minibuff, "DECLARE a1 CURSOR FOR SELECT W1.ent_date, W1.por_cislo, W2.desig, W1.md_acct, W1.md_anal, W1.md_amt, W1.d_acct, W1.d_anal, W1.d_amt FROM denik W1, descs W2 WHERE (W1.ent_date>='%s' AND W1.ent_date<='%s') AND W1.ent_date=W2.ent_date AND W1.por_cislo=W2.por_cislo AND (W1.md_acct='%s' OR W1.d_acct='%s') ORDER BY ent_date, por_cislo;", pgConvertDate(pgBeginDatum, start_date), pgConvertDate(pgEndDatum, end_date), acctnum, acctnum);
  } else {
    sprintf(minibuff, "DECLARE a1 CURSOR FOR SELECT W1.ent_date, W1.por_cislo, W2.desig, W1.md_acct, W1.md_anal, W1.md_amt, W1.d_acct, W1.d_anal, W1.d_amt FROM denik W1, descs W2 WHERE (W1.ent_date>='%s' AND W1.ent_date<='%s') AND W1.ent_date=W2.ent_date AND W1.por_cislo=W2.por_cislo AND ((W1.md_acct='%s' AND W1.md_anal='%s') OR (W1.d_acct='%s' AND W1.d_anal='%s')) ORDER BY ent_date, por_cislo;", pgConvertDate(pgBeginDatum, start_date), pgConvertDate(pgEndDatum, end_date), acctnum, analnum, acctnum, analnum);
  }
  startTransaction();
  runQuery(minibuff);
  printf("------------+--+------------------------------+-------------+-------------\n");
  printf("Datum:PorCis|An|Popis Transakce               |   MA DATI   |    DAL\n");
  printf("------------+--+------------------------------+-------------+-------------\n");
  while (fetchNext("a1")) {
    printf("%d:%03d|", (NathanConvertDate(getField("ent_date"))), atoi(getField("por_cislo")));
    desigBuff = strncpy(desigBuff, getField("desig"), 30);
    if ((amount = atoi(getField("md_amt"))) > 0) {
      printf("%s|%-30s|%10d.%02d|\n", getField("md_anal"), desigBuff, (amount/100), (amount%100));
      md_counter = md_counter + amount;
    } else 
      if ((amount = atoi(getField("d_amt"))) > 0) {
	printf("%s|%-30s|             |%10d.%02d\n", getField("d_anal"), desigBuff, (amount/100), (amount%100));
	d_counter = d_counter + amount;
      } else {
	fprintf(stderr, "Corrupt database: either credit or debit side must = 0 in 'denik'");
	exit_nicely();
      }
  }
  printf("\nTOTALS:                                        %10d.%02d %10d.%02d\n", (md_counter/100), (md_counter%100), (d_counter/100), (d_counter%100));
  if (d_counter < md_counter) {
    printf("                                 balance:                   (%10d.%02d)\n", ((md_counter - d_counter) / 100), ((md_counter - d_counter) % 100));
  } else 
    if (d_counter > md_counter) {
      printf("                                   balance:   (%10d.%02d)\n", ((d_counter - md_counter) / 100), ((d_counter - md_counter) % 100));
    }

  endTransaction();
  
  exit_nicely();
}
      
